function zh = Abstraction(z,k)
% Image Abstraction function
% input z: RGB image
% input k: filter iteration number
% output zh: filtered image

zcc = RGB2YCC(z);
l = squeeze(zcc(:,:,1));

[M,N] = size(l);

% sampling
max_sample = 125;
smp_ind = sampling (M,N, max_sample);

% smoothing parameter
h = 5;

% Nystrom
[phi,Pi] = Nyst(l,h,smp_ind,'spt');

% Sinkhorn
[W_A, W_B] = Sink(phi,Pi);
clear phi; clear Pi;

% Orthogonalizing
[V,lambda] = Orth(W_A,W_B);
clear W_A; clear W_B;

% permutation
V  = Perm(V,smp_ind);

% eigenvalue mapping
lambdak = lambda.^k;
lm0 = 30; %brightness correction
a1=1; a2=1; a3=2; a4=1.05;
lambdak(1:lm0) = a1 + (a2-a1)*lambda(1:lm0) + (a3-a2)*lambda(1:lm0).^2 + (a4-a3)*lambda(1:lm0).^3;

le = V*(lambdak.*(V'*l(:)));
zhcc = zcc;

le = reshape(le,M,N);
le(le>256)=255; le(le<0)=0;
zhcc(:,:,1) = le;
zh = YCC2RGB(zhcc);

end
