function zh = DetailManip(z,alpha)
% Detail manipulation function
% input z: RGB image
% input alpha: parameters (1x4)
% output zh: filtered image

zcc = RGB2YCC(z);
l = squeeze(zcc(:,:,1));

[M,N] = size(l);

% sampling
max_sample = 65;
smp_ind = sampling (M,N,max_sample);

% smoothing parameter
[gx gy] = gradient(l);
Ave_Grd = sum(sum(sqrt(gx.^2+gy.^2)))/(M*N);
h = 0.65*Ave_Grd;

% Nystrom
[phi,Pi] = Nyst(l,h,smp_ind,'nsp');

% Sinkhorn
[W_A, W_B] = Sink(phi,Pi);
clear phi; clear Pi;

% Orthogonalizing
[V,lambda] = Orth(W_A,W_B);
clear W_A; clear W_B;

% permutation
V  = Perm(V,smp_ind);

% projection
b = V'*l(:);
beta = sum(b.^2)/sum(l(:).^2)

% eigenvalue mapping
lambdak = alpha(1) + (alpha(2)-alpha(1))*lambda + (alpha(3)-alpha(2))*lambda.^2 + (alpha(4)-alpha(3))*lambda.^3;
le = V*(lambdak.*(V'*l(:)));
zhcc = zcc;

le = reshape(le,M,N);
le(le>256)=255; le(le<0)=0;
zhcc(:,:,1) = le;
zh = YCC2RGB(zhcc);

end
