function [zh, GlbMask] = GlobalMask(z,alpha,t)
% Globalizing mask for propagation
% input z: RGB image
% input alpha: parameters (1x4)
% input t: propagation parameter (diffusion parameter)
% output zh: filtered image
% output GlbMask: Globalizing Mask

zcc = RGB2YCC(z);
l = squeeze(zcc(:,:,1));

[M,N] = size(l);

% sampling
max_sample = 75;
smp_ind = sampling (M,N,max_sample);

% smoothing parameter
[gx gy] = gradient(l);
Ave_Grd = sum(sum(sqrt(gx.^2+gy.^2)))/(M*N);
h = 0.65*Ave_Grd;

% Nystrom
[phi,Pi] = Nyst(l,h,smp_ind,'nsp');

% Sinkhorn
[W_A, W_B] = Sink(phi,Pi);
clear phi; clear Pi;

% Orthogonalizing
[V,lambda] = Orth(W_A,W_B);
clear W_A; clear W_B;

% permutation
V  = Perm(V,smp_ind);

% projection
b = V'*l(:);
beta = sum(b.^2)/sum(l(:).^2)

% Globalizing Mask (diffusion map) %

p=size(V,2);
display('Please select a region!');
[Mask]=roipoly(uint8(z));%select a region
title('Input Image', 'FontSize', 12);
vMask = Mask(:);
[clx,crsx] = kmeans(V(vMask~=0,2:p),1);%k-means clustering
dist=zeros(M*N,1);
DifMask=dist;

%diffusion distance
for i=1:M*N
dist(i) = norm((lambda(2:p)'.^t).*(crsx-V(i,2:p)))^2;
DifMask(i) = exp(-dist(i)/0.000001); 
end

GlbMask = reshape(DifMask,[M,N]);

% eigenvalue mapping
lambdak = alpha(1) + (alpha(2)-alpha(1))*lambda + (alpha(3)-alpha(2))*lambda.^2 + (alpha(4)-alpha(3))*lambda.^3;
le = V*(lambdak.*(V'*l(:)));
zhcc = zcc;
zhcc2 =zcc;
zhcc3 = zcc;

le = reshape(le,M,N);
le(le>256)=255; le(le<0)=0;
zhcc(:,:,1) = le; 
zh_glob = YCC2RGB(zhcc);
figure,imshow(uint8(zh_glob),[0 255]);
title('Global Filtering', 'FontSize', 12);

leM = l;
leM(Mask==1) = le(Mask==1);
zhcc2(:,:,1) = leM; 
zh_reg = YCC2RGB(zhcc2);
figure,imshow(uint8(zh_reg),[0 255]);
title('Filtered Region', 'FontSize', 12)

lee = le.*GlbMask + l.*(1-GlbMask);
zhcc3(:,:,1) = lee; 
zh = YCC2RGB(zhcc3);
%figure,imshow(uint8(zh3),[0 255]);
%title('propagated edit');

end
